=== modified file 'bzrlib/clone.py'
--- bzrlib/clone.py	2005-09-30 06:10:59 +0000
+++ bzrlib/clone.py	2005-09-30 07:38:34 +0000
@@ -65,8 +65,6 @@
     to_location -- The destination directory; must either exist and be 
         empty, or not exist, in which case it is created.
 
-    revno -- This revision should be the head of the new branch.
-
     basis_branch
         A local branch to copy revisions from, related to branch_from. 
         This is used when branching from a remote (slow) branch, and we have
@@ -76,15 +74,14 @@
     assert isinstance(to_location, basestring)
     if basis_branch is not None:
         note("basis_branch is not supported for fast weave copy yet.")
+    history = _get_truncated_history(branch_from, revision)
     if not os.path.exists(to_location):
         os.mkdir(to_location)
     branch_to = Branch.initialize(to_location)
     mutter("copy branch from %s to %s", branch_from, branch_to)
     branch_to.set_root_id(branch_from.get_root_id())
-    if revision is None:
-        revision = branch_from.last_revision()
+    branch_to.append_revision(*history)
     _copy_control_weaves(branch_from, branch_to)
-    _copy_revision_history(branch_from, branch_to)
     _copy_text_weaves(branch_from, branch_to)
     _copy_revision_store(branch_from, branch_to)
     build_working_dir(to_location)
@@ -93,6 +90,18 @@
     return branch_to
 
 
+
+def _get_truncated_history(branch_from, revision):
+    history = branch_from.revision_history()
+    if revision is None:
+        return history
+    try:
+        idx = history.index(revision)
+    except ValueError:
+        raise InvalidRevisionId('revision {%s} is not on the mainline of %s' 
+                                % (revision, branch_from))
+    return history[:idx+1]
+
 def _copy_text_weaves(branch_from, branch_to):
     # TODO: Handle UnlistableStore and fall back to getting a list of 
     # all file-ids and copying them one by one.
@@ -104,11 +113,6 @@
     copy_all(branch_from.revision_store, branch_to.revision_store)
 
 
-def _copy_revision_history(branch_from, branch_to):
-    ff = branch_from.controlfile('revision-history')
-    tf = branch_to.put_controlfile('revision-history', ff.read())
-
-
 def _copy_control_weaves(branch_from, branch_to):
     to_control = branch_to.control_weaves
     from_control = branch_from.control_weaves

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-09-30 06:10:59 +0000
+++ bzrlib/selftest/testbranch.py	2005-09-30 07:38:34 +0000
@@ -86,6 +86,21 @@
         ## # basis branches currently disabled for weave format
         ## self.assertFalse(br_b.last_revision() in br_c.revision_history())
         ## br_c.get_revision(br_b.last_revision())
+
+    def test_copy_partial(self):
+        """Copy only part of the history of a branch."""
+        self.build_tree(['a/', 'a/one'])
+        br_a = Branch.initialize('a')
+        br_a.add(['one'])
+        br_a.commit('commit one', rev_id='u@d-1')
+        self.build_tree(['a/two'])
+        br_a.add(['two'])
+        br_a.commit('commit two', rev_id='u@d-2')
+        br_b = copy_branch(br_a, 'b', revision='u@d-1')
+        self.assertEqual(br_b.last_revision(), 'u@d-1')
+        self.assertTrue(os.path.exists('b/one'))
+        self.assertFalse(os.path.exists('b/two'))
+        
         
 # TODO: rewrite this as a regular unittest, without relying on the displayed output        
 #         >>> from bzrlib.commit import commit

